@@ -11,7 +11,7 @@ use pyo3::{
     PyClass, PyTypeInfo,
 };
 use serde::{Deserialize, Serialize};
-use serde_with::{serde_as, DisplayFromStr, FromInto};
+use serde_with::{serde_as, DisplayFromStr, FromInto, OneOrMany, TryFromInto};
 use solana_sdk::{
     clock::{Epoch, Slot, UnixTimestamp},
     epoch_info::EpochInfo as EpochInfoOriginal,
@@ -47,7 +47,7 @@ use crate::{
     pubkey::Pubkey,
     py_from_bytes_general_via_bincode, pybytes_general_via_bincode,
     signature::Signature,
-    tmp_account_decoder::{UiAccount, UiTokenAmount as UiTokenAmountOriginal},
+    tmp_account_decoder::{UiAccount, UiAccountData, UiTokenAmount as UiTokenAmountOriginal},
     tmp_transaction_status::{
         TransactionConfirmationStatus as TransactionConfirmationStatusOriginal,
         TransactionStatus as TransactionStatusOriginal, UiTransactionReturnData,
@@ -285,7 +285,7 @@ response_data_boilerplate!(RpcResponseContext);
 
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]
 #[serde(untagged)]
-pub enum Resp<T: PyClass + IntoPy<PyObject>> {
+pub enum Resp<T: IntoPy<PyObject>> {
     Result {
         #[serde(skip_deserializing)]
         jsonrpc: crate::rpc::requests::V2,
@@ -395,6 +395,16 @@ impl IntoPy<PyObject> for WebsocketMessage {
     }
 }
 
+#[serde_as]
+#[derive(Serialize, Deserialize, Clone, Debug, PartialEq)]
+pub struct WebsocketMessages(#[serde_as(deserialize_as = "OneOrMany<_>")] Vec<WebsocketMessage>);
+
+impl IntoPy<PyObject> for WebsocketMessages {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        self.0.into_py(py)
+    }
+}
+
 macro_rules! contextful_struct_def_eq {
     ($name:ident, $inner:ty) => {
         #[derive(Debug, PartialEq, Eq, Serialize, Deserialize, Clone)]
@@ -641,13 +651,19 @@ macro_rules! notification_contextless {
 contextful_resp_eq!(
     GetAccountInfoResp,
     Option<Account>,
-    "Option<FromInto<UiAccount>>"
+    "Option<TryFromInto<UiAccount>>"
 );
 
 contextful_resp_eq!(
     GetAccountInfoJsonParsedResp,
     Option<AccountJSON>,
-    "Option<FromInto<UiAccount>>"
+    "Option<TryFromInto<UiAccount>>"
+);
+
+contextful_resp_eq!(
+    GetAccountInfoMaybeJsonParsedResp,
+    Option<AccountMaybeJSON>,
+    "Option<TryFromInto<UiAccount>>"
 );
 
 contextful_resp_eq!(GetBalanceResp, u64);
@@ -971,7 +987,7 @@ pub struct RpcSimulateTransactionResult {
     pub err: Option<TransactionErrorType>,
     #[pyo3(get)]
     pub logs: Option<Vec<String>>,
-    #[serde_as(as = "Option<Vec<Option<FromInto<UiAccount>>>>")]
+    #[serde_as(as = "Option<Vec<Option<TryFromInto<UiAccount>>>>")]
     #[pyo3(get)]
     pub accounts: Option<Vec<Option<Account>>>,
     #[pyo3(get)]
@@ -1244,7 +1260,9 @@ contextful_resp_eq!(GetLargestAccountsResp, Vec<RpcAccountBalance>);
 #[pyclass(module = "solders.rpc.responses", subclass)]
 pub struct RpcBlockhash {
     #[serde_as(as = "DisplayFromStr")]
+    #[pyo3(get)]
     pub blockhash: SolderHash,
+    #[pyo3(get)]
     pub last_valid_block_height: u64,
 }
 
@@ -1280,12 +1298,87 @@ contextless_resp_eq!(GetMinimumBalanceForRentExemptionResp, u64);
 contextful_resp_eq!(
     GetMultipleAccountsResp,
     Vec<Option<Account>>,
-    "Vec<Option<FromInto<UiAccount>>>"
+    "Vec<Option<TryFromInto<UiAccount>>>"
 );
 contextful_resp_eq!(
     GetMultipleAccountsJsonParsedResp,
     Vec<Option<AccountJSON>>,
-    "Vec<Option<FromInto<UiAccount>>>"
+    "Vec<Option<TryFromInto<UiAccount>>>"
+);
+
+#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, Eq, FromPyObject)]
+#[serde(untagged)]
+pub enum AccountMaybeJSON {
+    Binary(Account),
+    Parsed(AccountJSON),
+}
+
+impl IntoPy<PyObject> for AccountMaybeJSON {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self {
+            Self::Parsed(x) => x.into_py(py),
+            Self::Binary(x) => x.into_py(py),
+        }
+    }
+}
+
+impl From<Account> for AccountMaybeJSON {
+    fn from(a: Account) -> Self {
+        Self::Binary(a)
+    }
+}
+
+impl From<AccountJSON> for AccountMaybeJSON {
+    fn from(a: AccountJSON) -> Self {
+        Self::Parsed(a)
+    }
+}
+
+impl TryFrom<AccountMaybeJSON> for Account {
+    type Error = String;
+    fn try_from(acc: AccountMaybeJSON) -> Result<Self, Self::Error> {
+        if let AccountMaybeJSON::Binary(account) = acc {
+            Ok(account)
+        } else {
+            Err("Expected Account, found AccountJSON".to_string())
+        }
+    }
+}
+
+impl TryFrom<AccountMaybeJSON> for AccountJSON {
+    type Error = String;
+    fn try_from(acc: AccountMaybeJSON) -> Result<Self, Self::Error> {
+        if let AccountMaybeJSON::Parsed(account) = acc {
+            Ok(account)
+        } else {
+            Err("Expected AccountJSON, found Account".to_string())
+        }
+    }
+}
+
+impl From<UiAccount> for AccountMaybeJSON {
+    fn from(u: UiAccount) -> Self {
+        match u.data {
+            UiAccountData::LegacyBinary(_) => panic!("LegacyBinary data should not appear"),
+            UiAccountData::Json(_) => AccountJSON::try_from(u).unwrap().into(),
+            UiAccountData::Binary(..) => Account::try_from(u).unwrap().into(),
+        }
+    }
+}
+
+impl From<AccountMaybeJSON> for UiAccount {
+    fn from(a: AccountMaybeJSON) -> Self {
+        match a {
+            AccountMaybeJSON::Binary(acc) => Self::from(acc),
+            AccountMaybeJSON::Parsed(acc) => Self::try_from(acc).unwrap(),
+        }
+    }
+}
+
+contextful_resp_eq!(
+    GetMultipleAccountsMaybeJsonParsedResp,
+    Vec<Option<AccountMaybeJSON>>,
+    "Vec<Option<TryFromInto<UiAccount>>>"
 );
 
 // the one in solana_client uses UiAccount from account_decoder which currently isn't portable
@@ -1297,7 +1390,7 @@ pub struct RpcKeyedAccount {
     #[serde_as(as = "DisplayFromStr")]
     #[pyo3(get)]
     pub pubkey: Pubkey,
-    #[serde_as(as = "FromInto<UiAccount>")]
+    #[serde_as(as = "TryFromInto<UiAccount>")]
     #[pyo3(get)]
     pub account: Account,
 }
@@ -1322,7 +1415,7 @@ pub struct RpcKeyedAccountJsonParsed {
     #[serde_as(as = "DisplayFromStr")]
     #[pyo3(get)]
     pub pubkey: Pubkey,
-    #[serde_as(as = "FromInto<UiAccount>")]
+    #[serde_as(as = "TryFromInto<UiAccount>")]
     #[pyo3(get)]
     pub account: AccountJSON,
 }
@@ -1345,14 +1438,36 @@ contextful_resp_eq!(
     Vec<RpcKeyedAccountJsonParsed>
 );
 
+contextless_resp_eq!(GetProgramAccountsResp, Vec<RpcKeyedAccount>, clone);
 contextless_resp_eq!(
-    GetProgramAccountsWithoutContextResp,
-    Vec<RpcKeyedAccount>,
+    GetProgramAccountsJsonParsedResp,
+    Vec<RpcKeyedAccountJsonParsed>,
     clone
 );
+
+#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, Eq, FromPyObject)]
+#[serde(untagged)]
+pub enum RpcKeyedAccountMaybeJSON {
+    Binary(RpcKeyedAccount),
+    Parsed(RpcKeyedAccountJsonParsed),
+}
+
+impl IntoPy<PyObject> for RpcKeyedAccountMaybeJSON {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self {
+            Self::Parsed(x) => x.into_py(py),
+            Self::Binary(x) => x.into_py(py),
+        }
+    }
+}
+
+contextful_resp_eq!(
+    GetProgramAccountsWithContextMaybeJsonParsedResp,
+    Vec<RpcKeyedAccountMaybeJSON>
+);
 contextless_resp_eq!(
-    GetProgramAccountsWithoutContextJsonParsedResp,
-    Vec<RpcKeyedAccountJsonParsed>,
+    GetProgramAccountsMaybeJsonParsedResp,
+    Vec<RpcKeyedAccountMaybeJSON>,
     clone
 );
 
@@ -2335,11 +2450,11 @@ contextless_resp_eq!(RequestAirdropResp, Signature, "DisplayFromStr");
 contextless_resp_eq!(SendTransactionResp, Signature, "DisplayFromStr");
 contextful_resp_eq!(SimulateTransactionResp, RpcSimulateTransactionResult);
 
-notification!(AccountNotification, Account, "FromInto<UiAccount>");
+notification!(AccountNotification, Account, "TryFromInto<UiAccount>");
 notification!(
     AccountNotificationJsonParsed,
     AccountJSON,
-    "FromInto<UiAccount>"
+    "TryFromInto<UiAccount>"
 );
 notification_no_eq!(BlockNotification, RpcBlockUpdate);
 notification!(LogsNotification, RpcLogsResponse);
@@ -2386,24 +2501,35 @@ impl IntoPy<PyObject> for ProgramNotificationType {
 contextless_resp_eq!(SubscriptionResp, u64);
 
 macro_rules ! pyunion_resp {
-    ($name:ident, $($variant:ident),+) => {
+    ($name:ident, $err_variant:ident, $($variant:ident),+) => {
         #[derive(FromPyObject, Clone, Debug, PartialEq, Serialize, Deserialize)]
         #[serde(untagged, rename_all = "camelCase")]
         pub enum $name {
+            $err_variant($err_variant),
             $($variant($variant),)+
         }
 
         impl $name {
             fn to_json(&self) -> String {
                 match self {
+                    Self::$err_variant(x) => x.py_to_json(),
                     $(Self::$variant(x) => x.py_to_json(),)+
                 }
             }
+
+            fn from_json(raw: &str, parser: &str) -> PyResult<Self> {
+                match parser {
+                    stringify!($err_variant) => {let parsed = $err_variant::py_from_json(raw)?; let as_enum = Self::RpcError(parsed); Ok(as_enum)},
+                    $(stringify!($variant) => {let parsed = $variant::py_from_json(raw)?; let as_enum = match parsed {Resp::Error {error, ..} => Self::RpcError(error), Resp::Result {result, ..} => Self::$variant(result)};Ok(as_enum)},)+
+                    _ => Err(PyValueError::new_err(format!("Unrecognised parser: {}", parser)))
+                }
+            }
         }
 
         impl IntoPy<PyObject> for $name {
             fn into_py(self, py: Python<'_>) -> PyObject {
                 match self {
+                    Self::$err_variant(x) => x.into_py(py),
                     $(Self::$variant(x) => x.into_py(py),)+
                 }
             }
@@ -2416,6 +2542,7 @@ pyunion_resp!(
     RpcError,
     GetAccountInfoResp,
     GetAccountInfoJsonParsedResp,
+    GetAccountInfoMaybeJsonParsedResp,
     GetBalanceResp,
     GetBlockProductionResp,
     GetBlockResp,
@@ -2444,10 +2571,13 @@ pyunion_resp!(
     GetMinimumBalanceForRentExemptionResp,
     GetMultipleAccountsResp,
     GetMultipleAccountsJsonParsedResp,
+    GetMultipleAccountsMaybeJsonParsedResp,
     GetProgramAccountsWithContextResp,
-    GetProgramAccountsWithoutContextResp,
+    GetProgramAccountsResp,
     GetProgramAccountsWithContextJsonParsedResp,
-    GetProgramAccountsWithoutContextJsonParsedResp,
+    GetProgramAccountsJsonParsedResp,
+    GetProgramAccountsWithContextMaybeJsonParsedResp,
+    GetProgramAccountsMaybeJsonParsedResp,
     GetRecentPerformanceSamplesResp,
     GetSignaturesForAddressResp,
     GetSignatureStatusesResp,
@@ -2466,7 +2596,6 @@ pyunion_resp!(
     GetTransactionResp,
     GetTransactionCountResp,
     GetVersionResp,
-    RpcVersionInfo,
     GetVoteAccountsResp,
     IsBlockhashValidResp,
     MinimumLedgerSlotResp,
@@ -2502,9 +2631,10 @@ pub fn batch_to_json(resps: Vec<RPCResult>) -> String {
 ///
 /// Args:
 ///     raw (str): The batch JSON string.
+///     parsers (Sequence): The classes to parse.
 ///
 /// Returns:
-///     A list of request objects.
+///     A list of response objects.
 ///
 /// Example:
 ///     >>> from solders.rpc.responses import batch_from_json, GetBlockHeightResp, GetFirstAvailableBlockResp
@@ -2526,18 +2656,10 @@ pub fn batch_from_json(raw: &str, parsers: Vec<&PyType>) -> PyResult<Vec<PyObjec
         let msg = format!("Number of parsers does not match number of response objects. Num parsers: {}. Num responses: {}", parsers_len, raw_objects_len);
         Err(PyValueError::new_err(msg))
     } else {
-        Python::with_gil(|py| {
-            Ok(raw_objects
-                .iter()
-                .zip(parsers.iter())
-                .map(|(res, parser)| {
-                    parser
-                        .call_method1("from_json", (serde_json::to_string(res).unwrap(),))
-                        .unwrap()
-                        .into_py(py)
-                })
-                .collect())
-        })
+        let parsed = raw_objects.iter().zip(parsers.iter()).map(|(res, parser)| {
+            RPCResult::from_json(&serde_json::to_string(res).unwrap(), parser.name().unwrap())
+        });
+        Python::with_gil(|py| parsed.map(|obj| obj.map(|o| o.into_py(py))).collect())
     }
 }
 
@@ -2563,7 +2685,7 @@ pub fn parse_notification(msg: &str) -> PyResult<Notification> {
     serde_json::from_str(msg).map_err(to_py_err)
 }
 
-/// Parse a message received by a Solana websocket subscription.
+/// Parse a message or array of messages received by a Solana websocket subscription.
 ///
 /// Args:
 ///     msg (str): The raw message JSON.
@@ -2575,13 +2697,24 @@ pub fn parse_notification(msg: &str) -> PyResult<Notification> {
 ///     >>> from solders.rpc.responses import parse_websocket_message
 ///     >>> raw = '{ "jsonrpc": "2.0", "method": "rootNotification", "params": { "result": 4, "subscription": 0 } }'
 ///     >>> parse_websocket_message(raw)
-///     RootNotification {
+///     [RootNotification {
 ///         result: 4,
 ///         subscription: 0,
-///     }
+///     }]
+///     >>> raw_multi = '[{"jsonrpc": "2.0", "result": 0, "id": 1}, {"jsonrpc": "2.0", "result": 1, "id": 2}]'
+///     >>> parse_websocket_message(raw_multi)
+///     [SubscriptionResult {
+///         jsonrpc: TwoPointOh,
+///         id: 1,
+///         result: 0,
+///     }, SubscriptionResult {
+///         jsonrpc: TwoPointOh,
+///         id: 2,
+///         result: 1,
+///     }]
 ///
 #[pyfunction]
-pub fn parse_websocket_message(msg: &str) -> PyResult<WebsocketMessage> {
+pub fn parse_websocket_message(msg: &str) -> PyResult<WebsocketMessages> {
     serde_json::from_str(msg).map_err(to_py_err)
 }
 
@@ -2605,6 +2738,7 @@ pub(crate) fn create_responses_mod(py: Python<'_>) -> PyResult<&PyModule> {
             RpcError::type_object(py),
             GetAccountInfoResp::type_object(py),
             GetAccountInfoJsonParsedResp::type_object(py),
+            GetAccountInfoMaybeJsonParsedResp::type_object(py),
             GetBalanceResp::type_object(py),
             GetBlockProductionResp::type_object(py),
             GetBlockResp::type_object(py),
@@ -2633,10 +2767,13 @@ pub(crate) fn create_responses_mod(py: Python<'_>) -> PyResult<&PyModule> {
             GetMinimumBalanceForRentExemptionResp::type_object(py),
             GetMultipleAccountsResp::type_object(py),
             GetMultipleAccountsJsonParsedResp::type_object(py),
+            GetMultipleAccountsMaybeJsonParsedResp::type_object(py),
             GetProgramAccountsWithContextResp::type_object(py),
-            GetProgramAccountsWithoutContextResp::type_object(py),
+            GetProgramAccountsResp::type_object(py),
             GetProgramAccountsWithContextJsonParsedResp::type_object(py),
-            GetProgramAccountsWithoutContextJsonParsedResp::type_object(py),
+            GetProgramAccountsJsonParsedResp::type_object(py),
+            GetProgramAccountsMaybeJsonParsedResp::type_object(py),
+            GetProgramAccountsWithContextMaybeJsonParsedResp::type_object(py),
             GetRecentPerformanceSamplesResp::type_object(py),
             GetSignaturesForAddressResp::type_object(py),
             GetSignatureStatusesResp::type_object(py),
@@ -2712,6 +2849,7 @@ pub(crate) fn create_responses_mod(py: Python<'_>) -> PyResult<&PyModule> {
     m.add_class::<RpcError>()?;
     m.add_class::<GetAccountInfoResp>()?;
     m.add_class::<GetAccountInfoJsonParsedResp>()?;
+    m.add_class::<GetAccountInfoMaybeJsonParsedResp>()?;
     m.add_class::<GetBalanceResp>()?;
     m.add_class::<RpcBlockProduction>()?;
     m.add_class::<RpcBlockProductionRange>()?;
@@ -2752,12 +2890,15 @@ pub(crate) fn create_responses_mod(py: Python<'_>) -> PyResult<&PyModule> {
     m.add_class::<GetMinimumBalanceForRentExemptionResp>()?;
     m.add_class::<GetMultipleAccountsResp>()?;
     m.add_class::<GetMultipleAccountsJsonParsedResp>()?;
+    m.add_class::<GetMultipleAccountsMaybeJsonParsedResp>()?;
     m.add_class::<RpcKeyedAccount>()?;
     m.add_class::<RpcKeyedAccountJsonParsed>()?;
     m.add_class::<GetProgramAccountsWithContextResp>()?;
-    m.add_class::<GetProgramAccountsWithoutContextResp>()?;
+    m.add_class::<GetProgramAccountsResp>()?;
     m.add_class::<GetProgramAccountsWithContextJsonParsedResp>()?;
-    m.add_class::<GetProgramAccountsWithoutContextJsonParsedResp>()?;
+    m.add_class::<GetProgramAccountsJsonParsedResp>()?;
+    m.add_class::<GetProgramAccountsWithContextMaybeJsonParsedResp>()?;
+    m.add_class::<GetProgramAccountsMaybeJsonParsedResp>()?;
     m.add_class::<RpcPerfSample>()?;
     m.add_class::<GetRecentPerformanceSamplesResp>()?;
     m.add_class::<RpcConfirmedTransactionStatusWithSignature>()?;
